/*
 * EmailValidator - a utility class for e-mail address validation.
 *
 * Author: Bogdan 'bogdro' Drozdowski, bogdandr <at> op . pl
 *
 *    EmailValidator - an application for e-mail address validation.
 *    Copyright (C) 2012 Bogdan 'bogdro' Drozdowski, bogdandr <at> op . pl
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

package BogDroSoft.ev;

import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Hashtable;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

/**
 * The e-mail address validation class.
 * @author Bogdan Drozdowski, bogdandr AT op . pl
 */
public class EmailValidator
{
	public static final int EV_VALID = 0;
	public static final int EV_INVALID = 1;
	public static final int EV_UNKNOWN = 2;

	private String serverResponse;

	/**
	 * Validate the given e-mail address.
	 * @param addressToValidate The address to validate. Must be non-null.
	 * @param ourAddress The sender e-mail address to present to the server.
	 * @param ourDomain The sender domain address to present to the server.
	 * @return <code>EV_VALID=0</code> if the server accepted the given address
	 *	(the address seems to ba valid), <code>EV_INVALID=1</code> if the
	 *	server rejected the given address and <code>EV_UNKNOWN=2</code> if
	 *	the status of the address could not be verified.
	 * @throws NamingException when naming could not be initialized.
	 * @throws UnknownHostException when the host could not be found.
	 * @throws IOException if input/output error occurrs on the socket.
	 * @throws NullPointerException when the address to validate is null.
	 * @throws IllegalArgumentException when the address to validate has
	 *	invalid syntax ('@' is missing).
	 * @throws UnsupportedEncodingException when the provided character set name is invalid.
	 */
	public int validate (String addressToValidate, String ourAddress, String ourDomain)
		throws NamingException, UnknownHostException, IOException, UnsupportedEncodingException
	{
		return validate (addressToValidate, ourAddress, ourDomain, null);
	}

	/**
	 * Validate the given e-mail address.
	 * @param addressToValidate The address to validate. Must be non-null.
	 * @param ourAddress The sender e-mail address to present to the server.
	 * @param ourDomain The sender domain address to present to the server.
	 * @param charset The chararacter set to use when decoding server replies.
	 * @return <code>EV_VALID=0</code> if the server accepted the given address
	 *	(the address seems to ba valid), <code>EV_INVALID=1</code> if the
	 *	server rejected the given address and <code>EV_UNKNOWN=2</code> if
	 *	the status of the address could not be verified.
	 * @throws NamingException when naming could not be initialized.
	 * @throws UnknownHostException when the host could not be found.
	 * @throws IOException if input/output error occurrs on the socket.
	 * @throws NullPointerException when the address to validate is null.
	 * @throws IllegalArgumentException when the address to validate has
	 *	invalid syntax ('@' is missing).
	 * @throws UnsupportedEncodingException when the provided character set name is invalid.
	 */
	public int validate (String addressToValidate, String ourAddress, String ourDomain, String charset)
		throws NamingException, UnknownHostException, IOException, UnsupportedEncodingException
	{
		if ( addressToValidate == null )
		{
			throw new NullPointerException (RequestUtilities.NPEmsg);
		}
		if ( addressToValidate.indexOf ('@') == -1 )
		{
			throw new IllegalArgumentException (RequestUtilities.IAEmsg
				+ addressToValidate);
		}
		Hashtable<String, String> env = new Hashtable<String, String> ();
		env.put (RequestUtilities.namingFactoryPar, RequestUtilities.namingFactoryParValue);
		DirContext dirctx = new InitialDirContext (env);
		String domain = addressToValidate.split (RequestUtilities.atChar)[1];
		// lookup the MX (Mail eXchanger) DNS record:
		Attributes a = dirctx.getAttributes (domain, new String[] {RequestUtilities.dnsType});
		NamingEnumeration<? extends Attribute> ne = a.getAll ();
		if ( ne.hasMore () )
		{
			// mail exchangers found - use them
			while ( ne.hasMore () )
			{
				Attribute at = ne.next ();
				NamingEnumeration<?> all = at.getAll ();
				while ( all.hasMore () )
				{
					String next = all.next ().toString ();
					String mx = next.split (RequestUtilities.dnsRecordSep)[1];
					// check the result. If success or failure, return.
					// If unknown, proceed
					int res = EV_UNKNOWN;
					try
					{
						if ( charset != null )
						{
							res = validateWithHost (addressToValidate,
								ourAddress, ourDomain, mx, charset);
						}
						else
						{
							res = validateWithHost (addressToValidate,
								ourAddress, ourDomain, mx);
						}
					}
					catch (UnknownHostException ex)
					{
						// ignore exceptions - just try the next address
						res = EV_UNKNOWN;
					}
					catch (UnsupportedEncodingException ex)
					{
						// ignore exceptions - just try the next address
						res = EV_UNKNOWN;
					}
					catch (IOException ex)
					{
						// ignore exceptions - just try the next address
						res = EV_UNKNOWN;
					}
					if ( res == EV_VALID || res == EV_INVALID )
					{
						return res;
					}
				}
			}
		}
		else
		{
			// no mail exchangers found - use the domain of the address
			if ( charset != null )
			{
				return validateWithHost (addressToValidate, ourAddress,
					ourDomain, domain, charset);
			}
			else
			{
				return validateWithHost (addressToValidate, ourAddress,
					ourDomain, domain);
			}
		}
		return EV_UNKNOWN;
	}

	/**
	 * Validate the given e-mail address.
	 * @param addressToValidate The address to validate. Must be non-null.
	 * @param ourAddress The sender e-mail address to present to the server.
	 * @param ourDomain The sender domain address to present to the server.
	 * @param host The SMTP server to validate the address on.
	 * @return <code>EV_VALID=0</code> if the server accepted the given address
	 *	(the address seems to ba valid), <code>EV_INVALID=1</code> if the
	 *	server rejected the given address and <code>EV_UNKNOWN=2</code> if
	 *	the status of the address could not be verified.
	 * @throws UnknownHostException when the host could not be found.
	 * @throws IOException if input/output error occurrs on the socket.
	 * @throws UnsupportedEncodingException when the provided character set name is invalid.
	 */
	private int validateWithHost (String addressToValidate,
		String ourAddress, String ourDomain, String host)
		throws UnknownHostException, IOException, UnsupportedEncodingException
	{
		return validateWithHost (addressToValidate, ourAddress, ourDomain, host, null);
	}

	/**
	 * Validate the given e-mail address.
	 * @param addressToValidate The address to validate. Must be non-null.
	 * @param ourAddress The sender e-mail address to present to the server.
	 * @param ourDomain The sender domain address to present to the server.
	 * @param host The SMTP server to validate the address on.
	 * @param charset The chararacter set to use when decoding server replies.
	 * @return <code>EV_VALID=0</code> if the server accepted the given address
	 *	(the address seems to ba valid), <code>EV_INVALID=1</code> if the
	 *	server rejected the given address and <code>EV_UNKNOWN=2</code> if
	 *	the status of the address could not be verified.
	 * @throws UnknownHostException when the host could not be found.
	 * @throws IOException if input/output error occurrs on the socket.
	 * @throws UnsupportedEncodingException when the provided character set name is invalid.
	 */
	private int validateWithHost (String addressToValidate,
		String ourAddress, String ourDomain, String host, String charset)
		throws UnknownHostException, IOException, UnsupportedEncodingException
	{
		if ( host == null ) return EV_UNKNOWN;
		Socket s = new Socket (host, 25);
		InputStream in = s.getInputStream ();
		OutputStream out = s.getOutputStream ();
		byte[] r = new byte[1000];
		int read;

		read = in.read (r);
		if ( read == 0 )
		{
			// nothing read, return
			s.close ();
			in = null;
			out = null;
			s = null;
			serverResponse = RequestUtilities.empty;
			return EV_UNKNOWN;
		}
		if ( r[0] != '2' )
		{
			makeServerResponse (r, read, charset);
			// invalid welcome message, return
			return EV_UNKNOWN;
		}
		if ( ourDomain != null )
		{
			out.write (makeServerCommandBytes (RequestUtilities.smtpEHLOStart
				+ ourDomain + RequestUtilities.smtpLineEnd, charset));
		}
		else
		{
			out.write (makeServerCommandBytes (RequestUtilities.smtpDefaultEHLO,
				charset));
		}

		read = in.read (r);
		if ( read == 0 )
		{
			// nothing read, return
			s.close ();
			in = null;
			out = null;
			s = null;
			serverResponse = RequestUtilities.empty;
			return EV_UNKNOWN;
		}
		if ( r[0] != '2' )
		{
			// invalid EHLO response, try HELO
			if ( ourDomain != null )
			{
				out.write (makeServerCommandBytes (RequestUtilities.smtpHELOStart
					+ ourDomain + RequestUtilities.smtpLineEnd, charset));
			}
			else
			{
				out.write (makeServerCommandBytes (RequestUtilities.smtpDefaultHELO,
					charset));
			}

			read = in.read (r);
			if ( r[0] != '2' )
			{
				// invalid EHLO response and HELO response, return
				makeServerResponse (r, read, charset);
				return EV_UNKNOWN;
			}
		}

		if ( ourAddress != null )
		{
			out.write (makeServerCommandBytes (RequestUtilities.smtpMailFromBeginning
				+ ourAddress + RequestUtilities.smtpMailFromEnd, charset));
		}
		else
		{
			out.write (makeServerCommandBytes (RequestUtilities.smtpDefaultMailFrom,
				charset));
		}

		read = in.read (r);
		if ( read == 0 )
		{
			// nothing read, return
			s.close ();
			in = null;
			out = null;
			s = null;
			serverResponse = RequestUtilities.empty;
			return EV_UNKNOWN;
		}
		if ( r[0] != '2' )
		{
			// invalid MAIL response, return
			makeServerResponse (r, read, charset);
			return EV_UNKNOWN;
		}
		out.write (makeServerCommandBytes (RequestUtilities.smtpRcptToBeginning
			+ addressToValidate + RequestUtilities.smtpMailFromEnd, charset));
		read = in.read (r);
		out.write (makeServerCommandBytes (RequestUtilities.smtpQuit, charset));
		s.close ();
		in = null;
		out = null;
		s = null;
		if ( read == 0 )
		{
			// nothing read, return
			serverResponse = RequestUtilities.empty;
			return EV_UNKNOWN;
		}
		// check the server code here:
		makeServerResponse (r, read, charset);
		if ( r[0] == '2' )
		{
			return EV_VALID;
		}
		else if ( r[0] == '4' ) // temporary error code
		{
			return EV_UNKNOWN;
		}
		else
		{
			return EV_INVALID;
		}
	}

	/**
	 * Makes the server resopnse String from the received bytes.
	 * @param b the byte array with the last server resopnse.
	 * @param len the length of the <code>b</code> array.
	 * @param charset the character set to use
	 * @throws UnsupportedEncodingException when the provided character set name is invalid.
	 */
	private void makeServerResponse (byte[] b, int len, String charset)
		throws UnsupportedEncodingException
	{
		if ( charset == null || charset.isEmpty () )
		{
			serverResponse = new String (b, 0, len, RequestUtilities.defaultCharset);
		}
		else
		{
			serverResponse = new String (b, 0, len, charset);
		}
	}

	/**
	 * Creates a byte array with the command to send to the server.
	 * @param command The command to put in the array.
	 * @param charset the character set to use
	 * @return a byte array with the command.
	 * @throws UnsupportedEncodingException when the provided character set name is invalid.
	 */
	private byte[] makeServerCommandBytes (String command, String charset)
		throws UnsupportedEncodingException
	{
		if ( charset == null || charset.isEmpty () )
		{
			return command.getBytes (RequestUtilities.defaultCharset);
		}
		else
		{
			return command.getBytes (charset);
		}
	}

	/**
	 * Returns the last server resopnse.
	 * @return the last server resopnse.
	 */
	public String getServerResponse ()
	{
		return serverResponse;
	}
}
