/*
 * RequestUtilities - a utility class for HTTP requests.
 *
 * Author: Bogdan 'bogdro' Drozdowski, bogdandr <at> op . pl
 *
 *    EmailValidator - an application for e-mail address validation.
 *    Copyright (C) 2012 Bogdan 'bogdro' Drozdowski, bogdandr <at> op . pl
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

package BogDroSoft.ev;

import javax.servlet.ServletRequest;

/**
 * RequestUtilities - a utility class for HTTP requests.
 * @author Bogdan 'bogdro' Drozdowski, bogdandr <at> op . pl
 */
public class RequestUtilities
{
	/** The name of the parameter which holds the e-mail address to validate. */
	public static final String reqParNameEmail = "EmailValidator_Email";
	/** The name of the parameter which holds the our domain to present to the target SMTP server. */
	public static final String reqParNameOurDomain = "EmailValidator_OurDomain";
	/** The name of the parameter which holds the our e-mail address to present to the target SMTP server. */
	public static final String reqParNameOurEmail = "EmailValidator_OurEmail";
	/** The name of the parameter which holds the character set to use when communicating with the SMTP server. */
	public static final String reqParNameCharset = "EmailValidator_Charset";

	/** An empty string "singleton". */
	public static final String empty = "";
	/** The message when a NullPointerException is thrown. */
	public static final String NPEmsg = "validate:addressToValidate == null";
	/** The message when an IllegalArgumentException is thrown. */
	public static final String IAEmsg = "validate:addressToValidate: invalid syntax: ";
	/** The naming factory parameter name. */
	public static final String namingFactoryPar = "java.naming.factory.initial";
	/** The naming factory parameter value. */
	public static final String namingFactoryParValue = "com.sun.jndi.dns.DnsContextFactory";
	/** The "at" character. */
	public static final String atChar = "@";
	/** The DNS entry to look for (mail exchangers). */
	public static final String dnsType = "MX";
	/** The DNS entry separator. */
	public static final String dnsRecordSep = "\\s";

	/** The SMTP line ending. */
	public static final String smtpLineEnd = "\r\n";
	/** The SMTP EHLO beginning. */
	public static final String smtpEHLOStart = "EHLO ";
	/** The SMTP default EHLO command. */
	public static final String smtpDefaultEHLO = "EHLO localhost\r\n";
	/** The SMTP HELO beginning. */
	public static final String smtpHELOStart = "HELO ";
	/** The SMTP default HELO command. */
	public static final String smtpDefaultHELO = "HELO localhost\r\n";
	/** The SMTP MAIL FROM beginning. */
	public static final String smtpMailFromBeginning = "MAIL FROM:<";
	/** The SMTP MAIL FROM end. */
	public static final String smtpMailFromEnd = ">\r\n";
	/** The SMTP default MAIL FROM. */
	public static final String smtpDefaultMailFrom = "MAIL FROM:<root@localhost>\r\n";
	/** The SMTP RCPT TO beginning. */
	public static final String smtpRcptToBeginning = "RCPT TO:<";
	/** The SMTP QUIT command. */
	public static final String smtpQuit = "QUIT\r\n";

	/** The default character set to use when reading server replies. */
	public static final String defaultCharset = "US-ASCII";

	private static final String leftAngBr = "<";
	private static final String rightAngBr = ">";
	private static final String ampersand = "&";
	private static final String leftAngBrEnt = "&lt;";
	private static final String rightAngBrEnt = "&gt;";
	private static final String ampersandEnt = "&amp;";

	// this is a utility class
	private RequestUtilities() {}

	/**
	 * Gets the value of the given parameter or returns an empty String.
	 * @param req the request to get the parameter from.
	 * @param name the name of the parameter to get.
	 * @return the value of the given parameter or an empty String.
	 */
	public static String getParameter (ServletRequest req, String name)
	{
		if ( req == null || name == null ) return empty;
		String ret = req.getParameter (name);
		if ( ret == null ) return empty;
		return ret;
	}

	/**
	 * Tells if the given parameter was present in the request.
	 * @param req the request to check the parameter in for.
	 * @param name the name of the parameter to check.
	 * @return TRUE, if the parameter was present.
	 */
	public static boolean hasParameter (ServletRequest req, String name)
	{
		if ( req == null || name == null ) return false;
		return req.getParameter (name) != null;
	}

	/**
	 * Returns a HTML-safe version of the given String. This means replacing
	 * all "&lt;", "&gt;" and "&amp;" characters to their corresponding entities.
	 * @param s The String to correct.
	 * @return The corrected (HTML-safe) String.
	 */
	public static String makeHTMLSafe (String s)
	{
		if ( s == null ) return empty;
		return s.replaceAll (ampersand, ampersandEnt)
			.replaceAll (leftAngBr, leftAngBrEnt)
			.replaceAll (rightAngBr, rightAngBrEnt);
	}

}

